from holon.principles import Principle

level0 = Principle(
    name="RMM Level 0",
    reference="https://martinfowler.com/articles/richardsonMaturityModel.html#level0",
)
level1 = Principle(
    name="RMM Level 1",
    reference="https://martinfowler.com/articles/richardsonMaturityModel.html#level1",
)
level2 = Principle(
    name="RMM Level 2",
    reference="https://martinfowler.com/articles/richardsonMaturityModel.html#level2",
)
level3 = Principle(
    name="RMM Level 3",
    reference="https://martinfowler.com/articles/richardsonMaturityModel.html#level3",
)
