from enum import Enum
from holon.base import Convention

upper_case = Convention(name="Upper Case")
lower_case = Convention(name="Lower Case")
snake_cake = Convention(name="Snake Case")
camel_case = Convention(name="Camel Case")
kebab_case = Convention(name="Kebab Case")
screaming_snake_case = Convention(name="Screaming Snake Case")
