from holon.principles import Principle

SLIDE_URL = "http://amundsen.com/talks/2016-11-apistrat-wadm/2016-11-apistrat-wadm.pdf"

database_centric = Principle(name="Database Centric", reference=SLIDE_URL)
object_centric = Principle(name="Object Centric", reference=SLIDE_URL)
resource_centric = Principle(name="Resource Centric", reference=SLIDE_URL)
affordance_centric = Principle(name="Affordance Centric", reference=SLIDE_URL)
